-----------------------------------------------------------------------
--
--  File:        djgplibr.ads
--  Description: Interface to DJGPP library functions
--  Rev:         0.4
--  Date:        16/02/96
--  Author:      Jerry van Dijk
--  Mail:        jerry@jvdsys.nextjk.stuyts.nl
--
--  Copyright (c) Jerry van Dijk, 1996
--  Forelstraat 211
--  2037 KV  HAARLEM
--  THE NETHERLANDS
--  tel int + 31 23 540 1052
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------
--
--  This specification contains a minimal binding to the DJGPP 1.12m4
--  library, incorporating only those functions needed by the
--  VGA_Graphics package.
--  Note that it depents on libpc.a being linked in. This library
--  contains the needed PC specific functions,
--  This is probably not just DJGPP but also GNAT specific code.
--
-----------------------------------------------------------------------

with Interfaces;   use Interfaces;
with Interfaces.C; use Interfaces.C;

package DJGPP_Library is

   ------------
   --  LIBC  --
   ------------

   --  Simplified processor registers
   type go32_dpmi_registers is
      record
         di     : unsigned_16;
         di_hi  : unsigned_16;
         si     : unsigned_16;
         si_hi  : unsigned_16;
         bp     : unsigned_16;
         bp_hi  : unsigned_16;
         res    : unsigned_16;
         res_hi : unsigned_16;
         bx     : unsigned_16;
         bx_hi  : unsigned_16;
         dx     : unsigned_16;
         dx_hi  : unsigned_16;
         cx     : unsigned_16;
         cx_hi  : unsigned_16;
         ax     : unsigned_16;
         ax_hi  : unsigned_16;
         flags  : unsigned_16;
         es     : unsigned_16;
         ds     : unsigned_16;
         fs     : unsigned_16;
         gs     : unsigned_16;
         ip     : unsigned_16;
         cs     : unsigned_16;
         sp     : unsigned_16;
         ss     : unsigned_16;
      end record;
   pragma Convention(C, go32_dpmi_registers);

   -- Go32 info block
   type go32_info_block is
      record
         size_of_this_structure_in_bytes       : unsigned_32;
         linear_address_of_primary_screen      : unsigned_32;
         linear_address_of_secondary_screen    : unsigned_32;
         linear_address_of_transfer_buffer     : unsigned_32;
         size_of_transfer_buffer               : unsigned_32;
         pid                                   : unsigned_32;
         master_interrupt_controller_base      : unsigned_8;
         slave_interrupt_controller_base       : unsigned_8;
         selector_for_linear_memory            : unsigned_16;
         linear_address_of_stub_info_structure : unsigned_32;
         linear_address_of_original_psp        : unsigned_32;
         run_mode                              : unsigned_16;
         run_mode_info                         : unsigned_16;
      end record;
   pragma Convention(C, go32_info_block);

   --  Call a DOS/BIOS interrupt
   procedure go32_dpmi_simulate_int(Vector : in     unsigned_16;
                                    Regs   : in out go32_dpmi_registers);
   pragma Import(C, go32_dpmi_simulate_int, "_go32_dpmi_simulate_int");

   --  Get a conventional memory selector
   function go32_conventional_mem_selector return unsigned_16;
   pragma Import(C, go32_conventional_mem_selector,
                    "_go32_conventional_mem_selector");

   --  Tranfer data from conventional memory
   procedure dosmemget(Offset : in unsigned_32;
                       Length : in unsigned_32;
                       Buffer : char_array);
   pragma Import(C, dosmemget);

   --  Transfer data to conventional memory
   procedure dosmemput(Buffer : char_array;
                       Length : unsigned_32;
                       Offset : unsigned_32);
   pragma Import(C, dosmemput);

   --  Current G032 info
   Current_Go32_Info_Block : go32_info_block;
   pragma Import(C, Current_Go32_Info_Block, "_go32_info_block");

   -------------
   --  LIBPC  --
   -------------

   --  Write byte to output port
   procedure outportb(Port : in unsigned_16;
                      Data : in unsigned_8);
   pragma Import(C, outportb);

   ----------------
   --  FARPTR.C  --
   ----------------

   -- Peek a byte
   function farpeekb(Selector : in unsigned_16;
                     Offset   : in unsigned_32)
                     return unsigned_8;
   pragma Import(C, farpeekb, "_farpeekb");

   -- Poke a byte
   procedure farpokeb(Selector : in unsigned_16;
                      Offset   : in unsigned_32;
                      Value    : in unsigned_8);
   pragma Import(C, farpokeb, "_farpokeb");

   --  Preset peek/pook selector
   procedure farsetsel(Selector : in unsigned_16);
   pragma Import(C, farsetsel, "_farsetsel");

   --  Preset Peek
   function farnspeekb(Offset : in unsigned_32)
                       return unsigned_8;
   pragma Import(C, farnspeekb, "_farnspeekb");

   --  Preset Poke
   procedure farnspokeb(Offset : in unsigned_32;
                        Value  : in unsigned_8);
   pragma Import(C, farnspokeb, "_farnspokeb");

end DJGPP_Library;