-----------------------------------------------------------------------
--
--  File:        pckeyboa.adb
--  Description: Basic keyboard interface
--  Rev:         0.4
--  Date:        16/02/96
--  Author:      Jerry van Dijk
--  Mail:        jerry@jvdsys.nextjk.stuyts.nl
--
--  Copyright (c) Jerry van Dijk, 1996
--  Forelstraat 211
--  2037 KV  HAARLEM
--  THE NETHERLANDS
--  tel int + 31 23 540 1052
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------

with Interfaces;    use Interfaces;
with DJGPP_Library; use DJGPP_Library;

package body PC_Keyboard is

   ----------------------------------------
   --  Returns True if a key is waiting  --
   ----------------------------------------
   function Is_Key_Waiting return Boolean is
      Result : Boolean := False;
      Regs   : go32_dpmi_registers;
   begin
      --  Get keyboard status from BIOS, is supposed to work only
      --  with AT keyboards. Use ax := 16#0100# otherwise. The last
      --  has the disavantage that control-break is also registered.
      --  If a key has been pressed the zero flag is zero.
      Regs.ax := 16#1100#;
      Regs.ss := 0; Regs.sp := 0; Regs.flags := 0;
      go32_dpmi_simulate_int(16#16#, Regs);
      if (Regs.flags and 16#40#) = 0 then
         Result := True;
      end if;
      return Result;
   end Is_Key_Waiting;

   -------------------------------------------------------------
   --  Waits until a key is pressed and returns its scancode  --
   -------------------------------------------------------------
   function Get_Scancode return Keyboard_Scancode is
      Regs : go32_dpmi_registers;
   begin
      --  AFAIK this returns the most usual keyboard scancodes
      Regs.ax := 16#0000#;
      Regs.ss := 0; Regs.sp := 0; Regs.flags := 0;
      go32_dpmi_simulate_int(16#16#, Regs);
      return Keyboard_Scancode(Regs.ax);
   end Get_Scancode;

   ----------------------------------------------------------------
   --  Wait until key is pressed and returns it character value  --
   ----------------------------------------------------------------
   function Get_Char return Character is
   begin
      return Character'Val(unsigned_16(Get_Scancode) and 16#00FF#);
   end Get_Char;

end PC_Keyboard;
