
------------------------------------------------------------
--        Name: Alden Dima
--      E-mail: dimaaa@seas.gwu.edu
--      School: The George Washington University
--              School of Engineering and Applied Science
--              Washington, D.C.
--       Class: CSci 298 - Independent Study
--     Project: Ada Curses Binding and Textual User Interface
--        File: cursmacr.adb "curses-macros.adb"
--        Date: 12/25/95
-- Description: package body for Ada implementation
--              of C-psuedofunctions (preprocessor macros)
--              in curses.h
------------------------------------------------------------

with interfaces.c;
use type interfaces.c.int;
with twiddle;
package body curses.macros is

--
-- bit-wise operations for chtype
--
   package twiddle_chtype is new 
      twiddle ( 
	 ada_int_type => chtype,
         c_int_type   => c.unsigned_int );  

   use twiddle_chtype;

--
-- #define addch( c )              waddch( stdscr, c )
--
   function addch ( ch : chtype ) return c.signed_int is
   begin
      return waddch ( stdscr, ch );
   end addch;

--
-- #define addrawch( c )           waddrawch( stdscr, c )
--
   function addrawch ( ch : chtype ) return c.signed_int is
   begin
      return waddrawch ( stdscr, ch );
   end addrawch;

--
-- #define box( w, v, h )          wborder( w, v, v, h, h, 0, 0, 0, 0 )
--
   function box (
      w : A_WINDOW_T;
      v : chtype;
      h : chtype )
      return c.signed_int is
   begin
      return wborder ( w, v, v, h, h, 0, 0, 0, 0 );
   end box;

--
-- #define cbreak()                (_cursvar.cbreak = TRUE)
--
   procedure cbreak is
   begin
      cursvar.cbreak := TRUE;
   end cbreak;

--
-- #define clear()                 (clearok( stdscr, TRUE )==ERR?ERR:wclear( stdscr ))
--
   function clear return c.signed_int is
   begin
      if clearok ( stdscr, curses.TRUE ) = curses.ERR then
	 return curses.ERR;
      else
	 return wclear ( stdscr );
      end if;
   end clear;

--
-- #define clrtobot()              wclrtobot( stdscr )
--
   function clrtobot return c.signed_int is
   begin
      return wclrtobot ( stdscr );
   end clrtobot;

--
-- #define clrtoeol()              wclrtoeol( stdscr )
--
   function clrtoeol return c.signed_int is
   begin
      return wclrtoeol ( stdscr );
   end clrtoeol;

--
-- #define COLOR_PAIR(n)  ((chtype)(n) << 24)
--
   function color_pair(n: chtype) return chtype is
   begin
      return lshift(n,24);
   end color_pair;

--
-- #define crmode()                (_cursvar.cbreak = TRUE)
--
   procedure crmode renames cbreak;

--
-- #define delch()                 wdelch( stdscr )
--
   function delch return c.signed_int is
   begin
      return wdelch ( stdscr );
   end delch;

--
-- #define deleteln()              wdeleteln( stdscr )
--
   function deleteln return c.signed_int is
   begin
      return wdeleteln ( stdscr );
   end deleteln;

--
-- #define echo()                  (_cursvar.echo = TRUE)
--
   procedure echo is
   begin
      cursvar.echo := curses.TRUE;
   end echo;

--
-- #define echochar(c)             (addch((chtype)c)==ERR?ERR:refresh())
--
   function echochar ( ch : chtype ) return c.signed_int is
   begin
      if addch ( ch ) = curses.ERR then
         return curses.ERR;
      else
         return refresh;    
      end if;
   end echochar;

--
-- #define erase()                 werase( stdscr )
--
   function erase return c.signed_int is
   begin
      return werase ( stdscr );
   end erase;

--
-- #define getbegyx(w,y,x)         ( y = (w)->_begy, x = (w)->_begx )
--
   procedure getbegyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int ) is
   begin
      y := w.begy;
      x := w.begx;
   end getbegyx;

--
-- #define getbegy(w)              (w)->_begy
--
   function getbegy ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return w.begy;
   end getbegy;

--
-- #define getch()                 wgetch(stdscr)
--
   function getch return c.signed_int is
   begin
      return wgetch ( stdscr );
   end getch;

--
-- #define getmaxx(w)              (w)->_maxx
--
   function getmaxx ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return w.maxx;
   end getmaxx;

--
-- #define getmaxy(w)              (w)->_maxy
--
   function getmaxy ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return w.maxy;
   end getmaxy;

--
-- #define getmaxyx(w,y,x)         ( y = (w)->_maxy, x = (w)->_maxx )
--
   procedure getmaxyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int ) is
   begin
      y := w.maxy;
      x := w.maxx;
   end getmaxyx;

--
-- #define getparx(w)              (w)->_parx
--
   function getparx ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return w.parx;
   end getparx;

--
-- #define getpary(w)              (w)->_pary
--
   function getpary ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return w.pary;
   end getpary;

--
-- #define getparyx(w,y,x)         ( y = (w)->_pary, x = (w)->_parx )
--
   procedure getparyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int ) is
   begin
      y := w.pary;
      x := w.parx;
   end getparyx;

--
-- #define getstr(str)             wgetstr( stdscr, str )
--
   function getstr ( str : c.charp ) return c.signed_int is
   begin
      return wgetstr ( stdscr, str );
   end getstr;

--
-- #define getyx(w,y,x)            ( y = (w)->_cury, x = (w)->_curx )
--
   procedure getyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int ) is
   begin
      y := w.cury;
      x := w.curx;
   end getyx;

--
-- #define keypad(w,flag)          (w->_use_keypad  = flag)
--
   procedure keypad (
      w    : A_WINDOW_T;
      flag : bool      ) is
   begin
      w.use_keypad := flag;
   end keypad;

--
-- #define leaveok(w,flag)         (w->_leave   = flag)
--
   procedure leaveok (
      w    : A_WINDOW_T;
      flag : bool      ) is
   begin
      w.leave := flag;
   end leaveok;

--
-- #define move(y,x)               wmove( stdscr, y, x )
--
   function move (
      y : c.signed_int;
      x : c.signed_int ) 
      return c.signed_int is
   begin
      return wmove ( stdscr, y, x );
   end move;

--
-- #define mvaddch(y,x,c)          (move( y, x )==ERR?ERR:addch( c ))
--
   function mvaddch (
      y : c.signed_int;
      x : c.signed_int;
      ch : chtype )
      return c.signed_int is
   begin
      if move ( y, x ) = ERR then
	 return ERR;
      else
	 return addch ( ch );
      end if;
   end mvaddch;

--
-- #define mvdelch(y,x)            (move( y, x )==ERR?ERR:wdelch( stdscr ))
--
   function mvdelch (
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int is
   begin
      if move ( y, x ) = ERR then
	 return ERR;
      else
	 return wdelch ( stdscr );
      end if;
   end mvdelch;

--
-- #define mvgetch(y,x)            (move( y, x )==ERR?ERR:wgetch(stdscr))
--
   function mvgetch (
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int is
   begin
      if move ( y, x ) = curses.ERR then
         return curses.ERR;
      else
         return wgetch ( stdscr );
      end if;
   end mvgetch;

--
-- #define mvgetstr(y,x,str)       (move( y, x )==ERR?ERR:wgetstr( stdscr, str ))
--
   function mvgetstr (
      y   : c.signed_int;
      x   : c.signed_int;
      str : c.charp )
      return c.signed_int is
   begin
      if move ( y, x ) = curses.ERR then
         return curses.ERR;
      else
         return wgetstr ( stdscr, str );
      end if;
   end mvgetstr;

--
-- #define mvwaddstr(w,y,x,str)    (wmove( w, y, x )==ERR?ERR:waddstr( w, str ))
--
   function mvwaddstr (
      w   : A_WINDOW_T;
      y   : c.signed_int;
      x   : c.signed_int;
      str : c.charp )
      return c.signed_int is
   begin
      if wmove ( w, y, x ) = ERR then
	 return ERR;
      else
	 return waddstr ( w, str );
      end if;
   end mvwaddstr;

--
-- #define mvwaddrawch(w,y,x,c)    (wmove( w, y, x )==ERR?ERR:waddrawch( w, c ))
--
   function mvwaddrawch (
      w  : A_WINDOW_T;
      y  : c.signed_int;
      x  : c.signed_int;
      ch : chtype )
      return c.signed_int is
   begin
      if wmove ( w, y, x ) = curses.ERR then
         return curses.ERR;
      else
         return waddrawch( w, ch );
      end if;
   end mvwaddrawch;

--
-- #define mvwgetch(w,y,x)         (wmove( w, y, x )==ERR?ERR:wgetch( w ))
--
   function mvwgetch (
      w : A_WINDOW_T;
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int is
   begin
      if wmove ( w, y, x ) = curses.ERR then
         return curses.ERR;
      else
         return wgetch ( w );
      end if;
   end mvwgetch;

--
-- #define mvwgetstr(w,y,x,str)    (wmove( w, y, x )==ERR?ERR:wgetstr( w, str ))
--
   function mvwgetstr (
      w   : A_WINDOW_T;
      y   : c.signed_int;
      x   : c.signed_int;
      str : c.charp )
      return c.signed_int is
   begin
      if wmove ( w, y, x ) = curses.ERR then
	 return curses.ERR;
      else
	 return wgetstr ( w, str );
      end if;
   end mvwgetstr;
      
--
-- #define nocbreak()              (_cursvar.cbreak = FALSE)
--
   procedure nocbreak is
   begin
      cursvar.cbreak := FALSE;
   end nocbreak;

--
-- #define nocrmode()              (_cursvar.cbreak = FALSE)
--
   procedure nocrmode renames nocbreak;

--
-- #define nodelay(w,flag)         (w->_nodelay = flag)
--
   procedure nodelay (
      w    : A_WINDOW_T;
      flag : bool      ) is
   begin
      w.nodelay := flag;
   end nodelay;

--
-- #define noecho()                (_cursvar.echo = FALSE)
--
   procedure noecho is
   begin
      cursvar.echo := FALSE;
   end noecho;

--
-- #define nl()                    (_cursvar.autocr = TRUE)
--
   procedure nl is
   begin
      cursvar.autocr := curses.TRUE;
   end nl;

--
-- #define nonl()                  (_cursvar.autocr = FALSE)
--
   procedure nonl is
   begin
      cursvar.autocr := curses.FALSE;
   end nonl;

--
-- #define scroll(w)               wscrl((w),1)
--
   function scroll ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return wscrl ( w, 1);
   end scroll;

--
-- #define standend()              wattrset(stdscr, A_NORMAL)
--
   function standend return c.signed_int is
   begin
      return wattrset ( stdscr, A_NORMAL );
   end standend;

--
-- #define standout()              wattrset(stdscr, A_STANDOUT)
--
   function standout return c.signed_int is
   begin
      return wattrset ( stdscr, A_STANDOUT );
   end standout;
   
--
-- #define scrollok(w,flag)        ((w)->_scroll  = flag)
--
   procedure scrollok (
      w    : A_WINDOW_T;
      flag : bool      ) is
   begin
      w.scroll := flag;
   end scrollok;

--
-- #define waddch(w, c)            PDC_chadd( w, (chtype)c, (bool)!(_cursvar.raw_out), TRUE )
--
   function waddch (
      w  : A_WINDOW_T;
      ch : chtype)
      return c.signed_int
   is
      temp : bool;
   begin
      if cursvar.raw_out = curses.TRUE then
	 temp := curses.FALSE;
      else
	 temp := curses.TRUE;
      end if;
      return PDC_CHADD ( w, ch, temp, curses.TRUE );
   end waddch;

--
-- #define waddrawch(w, c)         PDC_chadd( w, (chtype)c, FALSE, TRUE )
--
   function waddrawch (
      w  : A_WINDOW_T;
      ch : chtype )
      return c.signed_int is
   begin
      return PDC_CHADD ( w, ch, curses.FALSE, curses.TRUE );
   end waddrawch;

--
-- #define wclear(w)               ( werase( w )==ERR?ERR:((w)->_clear = TRUE))
--  
   function wclear ( w : A_WINDOW_T ) return c.signed_int is
   begin
      if werase ( w) = curses.ERR then
         return curses.ERR;
      else
         w.clear := curses.TRUE;
         return curses.TRUE;
      end if; 
   end wclear;

--
-- #define wstandend(w)            wattrset(w, A_NORMAL)
--
   function wstandend ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return wattrset ( w, A_NORMAL );
   end wstandend;

--
-- #define wstandout(w)            wattrset(w, A_STANDOUT)
--
   function wstandout ( w : A_WINDOW_T ) return c.signed_int is
   begin
      return wattrset(w, A_STANDOUT);
   end wstandout;

--
-- #define wechochar(w,c)          (waddch(w,(chtype)c)==ERR?ERR:wrefresh(w))
--
   function wechochar (
      w  : A_WINDOW_T;
      ch : chtype )
      return c.signed_int is
   begin
      if waddch ( w, ch ) = curses.ERR then
         return curses.ERR;
      else
         return wrefresh ( w );
      end if;
   end wechochar;

end curses.macros;
