
------------------------------------------------------------
--        Name: Alden Dima
--      E-mail: dimaaa@seas.gwu.edu
--      School: The George Washington University
--              School of Engineering and Applied Science
--              Washington, D.C.
--       Class: CSci 298 - Independent Study
--     Project: Ada Curses Binding and Textual User Interface
--        File: cursmacr.ads "curses-macros.ads"
--        Date: 12/25/95
-- Description: package specification for Ada implementation
--              of C-psuedofunctions (preprocessor macros)
--              in curses.h
------------------------------------------------------------

package curses.macros is
--
-- #define addch( c )  waddch( stdscr, c )
--
   function addch ( ch : chtype ) return c.signed_int;

--
-- #define addrawch( c )  waddrawch( stdscr, c )
--
   function addrawch ( ch : chtype ) return c.signed_int;

--
-- #define box( w, v, h )  wborder( w, v, v, h, h, 0, 0, 0, 0 )
--
   function box (
      w : A_WINDOW_T;
      v : chtype;
      h : chtype )
      return c.signed_int;

--
-- #define cbreak()  (_cursvar.cbreak = TRUE)
--
   procedure cbreak;

--
-- #define clear()  (clearok( stdscr, TRUE )==ERR?ERR:wclear( stdscr ))
--
   function clear return c.signed_int;

--
-- #define clrtobot()  wclrtobot( stdscr )
--
   function clrtobot return c.signed_int;

--
-- #define clrtoeol()  wclrtoeol( stdscr )
--
   function clrtoeol return c.signed_int;

--
-- #define COLOR_PAIR(n)  ((chtype)(n) << 24)
--
   function color_pair ( n: chtype ) return chtype;

--
-- #define crmode()  (_cursvar.cbreak = TRUE)
--
   procedure crmode;

--
-- #define delch()  wdelch( stdscr )
--
   function delch return c.signed_int;

--
-- #define deleteln()  wdeleteln( stdscr )
--
   function deleteln return c.signed_int;

--
-- #define echo()  (_cursvar.echo = TRUE)
--
   procedure echo;

--
-- #define echochar(c)  (addch((chtype)c)==ERR?ERR:refresh())
--
   function echochar ( ch : chtype ) return c.signed_int;

--
-- #define erase()  werase( stdscr )
--
   function erase return c.signed_int;

--
-- #define getbegyx(w,y,x)  ( y = (w)->_begy, x = (w)->_begx )
--
   procedure getbegyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int );

--
-- #define getbegy(w)  (w)->_begy
--
   function getbegy ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define getch()  wgetch(stdscr)
--
   function getch return c.signed_int;

--
-- #define getmaxx(w)  (w)->_maxx
--
   function getmaxx ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define getmaxy(w)  (w)->_maxy
--
   function getmaxy ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define getmaxyx(w,y,x)  ( y = (w)->_maxy, x = (w)->_maxx )
--
   procedure getmaxyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int );

--
-- #define getparx(w)  (w)->_parx
--
   function getparx ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define getpary(w)  (w)->_pary
--
   function getpary ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define getparyx(w,y,x)  ( y = (w)->_pary, x = (w)->_parx )
--
   procedure getparyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int );

--
-- #define getstr(str)  wgetstr( stdscr, str )
--  
   function getstr ( str : c.charp ) return c.signed_int;

--
-- #define getyx(w,y,x)  ( y = (w)->_cury, x = (w)->_curx )
--
   procedure getyx (
      w : in     A_WINDOW_T;
      y :    out c.signed_int;
      x :    out c.signed_int );

--
-- #define keypad(w,flag)  (w->_use_keypad  = flag)
--
   procedure keypad  (
      w    : A_WINDOW_T;
      flag : bool );

--
-- #define leaveok(w,flag)  (w->_leave   = flag)
--
   procedure leaveok (
      w    : A_WINDOW_T;
      flag : bool );

--
-- #define move(y,x)  wmove( stdscr, y, x )
--
   function move (
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int;

--
-- #define mvaddch(y,x,c)  (move( y, x )==ERR?ERR:addch( c ))
--
  function mvaddch (
     y  : c.signed_int;
     x  : c.signed_int;
     ch : chtype )
     return c.signed_int;

--
-- #define mvdelch(y,x)  (move( y, x )==ERR?ERR:wdelch( stdscr ))
--
   function mvdelch ( 
      y : c.signed_int;
      x : c.signed_int)
      return c.signed_int;

--
-- #define mvgetch(y,x)  (move( y, x )==ERR?ERR:wgetch(stdscr))
--
   function mvgetch (
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int;

--
-- #define mvgetstr(y,x,str)  (move( y, x )==ERR?ERR:wgetstr( stdscr, str ))
--
   function mvgetstr (
      y   : c.signed_int;
      x   : c.signed_int;
      str : c.charp )
      return c.signed_int;

--
-- #define mvwaddstr(w,y,x,str)  (wmove( w, y, x )==ERR?ERR:waddstr( w, str ))
--
   function mvwaddstr (
      w   : A_WINDOW_T; 
      y   : c.signed_int;
      x   : c.signed_int;
      str : c.charp )
      return c.signed_int;

--
-- #define mvwaddrawch(w,y,x,c)  (wmove( w, y, x )==ERR?ERR:waddrawch( w, c ))
--
   function mvwaddrawch (
      w  : A_WINDOW_T;
      y  : c.signed_int;
      x  : c.signed_int;
      ch : chtype )
      return c.signed_int;

--
-- #define mvwgetch(w,y,x)  (wmove( w, y, x )==ERR?ERR:wgetch( w ))
--
   function mvwgetch (
      w : A_WINDOW_T;
      y : c.signed_int;
      x : c.signed_int )
      return c.signed_int;

--
-- #define mvwgetstr(w,y,x,str)  (wmove( w, y, x )==ERR?ERR:wgetstr( w, str ))
--
     function mvwgetstr (
        w   : A_WINDOW_T;
        y   : c.signed_int;
        x   : c.signed_int;
        str : c.charp )
        return c.signed_int;

--
-- #define nl()  (_cursvar.autocr = TRUE)
--
   procedure nl;

--
-- #define nocbreak()  (_cursvar.cbreak = FALSE)
--
   procedure nocbreak;

--
-- #define nocrmode()  (_cursvar.cbreak = FALSE)
--
   procedure nocrmode;

--
-- #define nodelay(w,flag)  (w->_nodelay = flag)
--
   procedure nodelay (
      w    : A_WINDOW_T;
      flag : bool );

--
-- #define noecho()  (_cursvar.echo = FALSE)
--
   procedure noecho;

--
-- #define nonl()  (_cursvar.autocr = FALSE)
--
   procedure nonl;

--
-- #define scroll(w)  wscrl((w),1)
--
   function scroll ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define scrollok(w,flag)  ((w)->_scroll  = flag)
--
   procedure scrollok (
      w    : A_WINDOW_T;
      flag : bool );

--
-- #define standend()  wattrset(stdscr, A_NORMAL)
--
   function standend return c.signed_int;

--
-- #define standout()  wattrset(stdscr, A_STANDOUT)
--
   function standout return c.signed_int;

--
-- #define waddch(w, c)  PDC_chadd( w, (chtype)c, (bool)!(_cursvar.raw_out), TRUE )
--
   function waddch (
      w  : A_WINDOW_T;
      ch : chtype)
      return c.signed_int;

--
-- #define waddrawch(w, c)  PDC_chadd( w, (chtype)c, FALSE, TRUE )
--
   function waddrawch (
      w  : A_WINDOW_T;
      ch : chtype )
      return c.signed_int;

--
-- #define wclear(w)  ( werase( w )==ERR?ERR:((w)->_clear = TRUE))
--
   function wclear ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define wstandend(w)  wattrset(w, A_NORMAL)
--
   function wstandend ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define wstandout(w)  wattrset(w, A_STANDOUT)
--
   function wstandout ( w : A_WINDOW_T ) return c.signed_int;

--
-- #define wechochar(w,c)  (waddch(w,(chtype)c)==ERR?ERR:wrefresh(w))
--
   function wechochar (
      w  : A_WINDOW_T;
      ch : chtype )
      return c.signed_int;

end curses.macros;
