--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

package body BC.Queues.Bounded is

   procedure Clear  (Obj : in out Bnd_Queue) is
   begin
      Bnd_Queue_Nodes.Clear(Obj.Rep);
   end Clear;

   procedure Append (Obj : in out Bnd_Queue; Elem : Item) is
   begin
      Bnd_Queue_Nodes.Append(Obj.Rep, Elem);
   end Append;

   procedure Pop    (Obj : in out Bnd_Queue) is
   begin
      Bnd_Queue_Nodes.Remove(Obj.Rep, 1);
   end Pop;

   procedure Remove (Obj : in out Bnd_Queue; From : Natural) is
   begin
      Bnd_Queue_Nodes.Remove(Obj.Rep, From);
   end Remove;

   function Available (Obj: in Bnd_Queue) return Natural is
   begin
      return Bnd_Queue_Nodes.Available(Obj.Rep);
   end Available;

   function Length  (Obj : Bnd_Queue) return Natural is
   begin
      return Bnd_Queue_Nodes.Length(Obj.Rep);
   end Length;

   function Is_Empty(Obj : Bnd_Queue) return Boolean is
   begin
      return Bnd_Queue_Nodes.Length(Obj.Rep) = 0;
   end Is_Empty;

   function Front   (Obj : Bnd_Queue) return Item is
   begin
      return Bnd_Queue_Nodes.First(Obj.Rep);
   end Front;

   function Location(Obj : Bnd_Queue; Elem : Item) return Natural is
      use Bnd_Queue_Nodes;
      AObj      : aliased Bnd_Node := Obj.Rep;
   begin
      return Bnd_Queue_Nodes.Location(AObj'access, Elem);
   end Location;

   function "="(Left, Right : Bnd_Queue) return Boolean is
      use Bnd_Queue_Nodes;
   begin
      return Left.Rep = Right.Rep;
   end "=";

   procedure Purge(Obj : in out Bnd_Queue) is
   begin
      Bnd_Queue_Nodes.Clear(Obj.Rep);
   end Purge;

   procedure Add(Obj : in out Bnd_Queue; Elem : in out Item) is
   begin
      Bnd_Queue_Nodes.Append(Obj.Rep, Elem);
   end Add;

   function Cardinality(Obj : Bnd_Queue) return Integer is
   begin
      return Bnd_Queue_Nodes.Length(Obj.Rep);
   end Cardinality;

   function Item_At(Obj : Bnd_Queue; Index : Natural) return Item is
      use Bnd_Queue_Nodes;
      AObj      : aliased Bnd_Node := Obj.Rep;
   begin
      return Bnd_Queue_Nodes.Item_At(Aobj'access, Index);
   end Item_at;

   function Item_At(Obj : Bnd_Queue; Index : Natural) return Item_Ptr is
      use Bnd_Queue_Nodes;
      AObj     : aliased Bnd_Node := Obj.Rep;
      The_Item : aliased Item := Bnd_Queue_Nodes.Item_At(Aobj'access, Index);
      AItem  : Item_Ptr := The_Item'Unchecked_Access;
   begin
      return AItem;
   end Item_at;

end BC.Queues.Bounded;
