--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with Unchecked_Deallocation;
package body BC.Queues.Dynamic is

   function Create return Dyn_Queue is
      Temp : Dyn_Queue := (Controlled with Rep => Dyn_Queue_Nodes.Create);
   begin
      return Temp;
   end Create;

   function Create(Size : Positive) return Dyn_Queue is
      Temp : Dyn_Queue := (Controlled with Rep => Dyn_Queue_Nodes.Create(Size));
   begin
      return Temp;
   end Create;

   function "="(Left, Right : Dyn_Queue) return Boolean is
      use Dyn_Queue_Nodes;
   begin
      return Left.Rep = Right.Rep;
   end "=";

   procedure Clear  (Obj : in out Dyn_Queue) is
   begin
      Dyn_Queue_Nodes.Clear(Obj.Rep);
   end Clear;

   procedure Append (Obj : in out Dyn_Queue; Elem : Item) is
   begin
      Dyn_Queue_Nodes.Append(Obj.Rep, Elem);
   end Append;

   procedure Pop    (Obj : in out Dyn_Queue) is
   begin
      Dyn_Queue_Nodes.Remove(Obj.Rep, 1);
   end Pop;

   procedure Remove (Obj : in out Dyn_Queue; From : Natural) is
   begin
      Dyn_Queue_Nodes.Remove(Obj.Rep, From);
   end Remove;

   function Length  (Obj : Dyn_Queue) return Natural is
   begin
      return Dyn_Queue_Nodes.Length(Obj.Rep);
   end Length;

   function Is_Empty(Obj : Dyn_Queue) return Boolean is
   begin
      return Dyn_Queue_Nodes.Length(Obj.Rep) = 0;
   end Is_Empty;

   function Front   (Obj : Dyn_Queue) return Item is
   begin
      return Dyn_Queue_Nodes.First(Obj.Rep);
   end Front;

   function Location(Obj : Dyn_Queue; Elem : Item) return Natural is
   begin
      return Dyn_Queue_Nodes.Location(Obj.Rep, Elem);
   end Location;

   procedure Preallocate   (Obj : in out Dyn_Queue; Size : Natural) is
   begin
      Dyn_Queue_Nodes.Preallocate(Obj.Rep, Size);
   end Preallocate;

   procedure Set_Chunk_Size(Obj : in out Dyn_Queue; Size : Natural) is
   begin
      Dyn_Queue_Nodes.Set_Chunk_Size(Obj.Rep, Size);
   end Set_Chunk_Size;

   function Chunk_Size(Obj : Dyn_Queue) return Natural is
   begin
      return Dyn_Queue_Nodes.Chunk_Size(Obj.Rep);
   end Chunk_Size;

   procedure Purge(Obj : in out Dyn_Queue) is
   begin
      Dyn_Queue_Nodes.Clear(Obj.Rep);
   end Purge;

   procedure Add(Obj : in out Dyn_Queue; Elem : in out Item) is
   begin
      Dyn_Queue_Nodes.Append(Obj.Rep, Elem);
   end Add;

   function Cardinality(Obj : Dyn_Queue) return Integer is
   begin
      return Dyn_Queue_Nodes.Length(Obj.Rep);
   end Cardinality;

   function Item_At(Obj : Dyn_Queue; Index : Natural) return Item is
   begin
      return Dyn_Queue_Nodes.Item_At(Obj.Rep, Index);
   end Item_at;

   function Item_At(Obj : Dyn_Queue; Index : Natural) return Item_Ptr is
      use Dyn_Queue_Nodes;
      The_Item : aliased Item := Dyn_Queue_Nodes.Item_At(Obj.Rep, Index);
      AItem  : Item_Ptr := The_Item'Unchecked_Access;
   begin
      return AItem;
   end Item_at;

end BC.Queues.Dynamic;
