--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with BC.Support.Dynamic;
generic
package BC.Queues.Dynamic is

   type Dyn_Queue is new Queue with private;
	-- A dynamic Queue exhibits similar performance to a Bounded_Queue,
	-- except its size is limited only to available memory.  It dynamically
	-- grows in a linear fashion (based on Chunk_Size).  There is currently
	-- no support for linear collapsing of the Queue.

   function Create(Size : Positive) return Dyn_Queue;
	-- Creates a new Dynamic Queue that is preallocated for 'Size' elements

   function "="(Left, Right : Dyn_Queue) return Boolean;
   procedure Clear  (Obj : in out Dyn_Queue);
   procedure Append (Obj : in out Dyn_Queue; Elem : Item);
   procedure Pop    (Obj : in out Dyn_Queue);
   procedure Remove (Obj : in out Dyn_Queue; From : Natural);
   function Length  (Obj : Dyn_Queue) return Natural;
   function Is_Empty(Obj : Dyn_Queue) return Boolean;
   function Front   (Obj : Dyn_Queue) return Item;
   function Location(Obj : Dyn_Queue; Elem : Item) return Natural;

   procedure Preallocate   (Obj : in out Dyn_Queue; Size : Natural);
	-- Allocates 'Size' additional storage elements for the Queue

   procedure Set_Chunk_Size(Obj : in out Dyn_Queue; Size : Natural);
	-- Establishes the Size the Queue will grow if the Queue exhausts its
	-- current size.

   function Chunk_Size(Obj : Dyn_Queue) return Natural;
	-- Returns the Chunk_Size

   procedure Purge(Obj : in out Dyn_Queue);
   procedure Add(Obj : in out Dyn_Queue; Elem : in out Item);
   function Cardinality(Obj : Dyn_Queue) return Integer;
   function Item_At(Obj : Dyn_Queue; Index : Natural) return Item;
   function Item_At(Obj : Dyn_Queue; Index : Natural) return Item_Ptr;


private
   package Dyn_Queue_Nodes is new BC.Support.Dynamic(Item);

   type Dyn_Queue is new Queue with record
      Rep : Dyn_Queue_Nodes.Dyn_Node;
   end record;

end BC.Queues.Dynamic;
