--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

package body BC.Support.Bounded is

   procedure Clear  (Obj : in out Bnd_Node) is
   begin
      Obj.Size := 0;
   end Clear;

   procedure Insert(Obj : in out Bnd_Node; Elem : Item) is
   begin
      pragma Assert(Obj.Size < Max_Size, "Attempt to insert into full array");
      Obj.Elems(2..Obj.Size + 1) := Obj.Elems(1..Obj.Size);
      Obj.Elems(1) := Elem;
      Obj.Size := Obj.Size + 1;
   end Insert;

   procedure Insert(Obj : in out Bnd_Node; Elem : Item; Before : Natural) is
   begin
      pragma Assert(Obj.Size < Max_Size, "Attempt to insert into full array");
      if (Obj.Size = 0) or else (Before = 1) then
         Insert(Obj,Elem);
      else
     Obj.Elems(Before + 1 .. Obj.Size + 1) := Obj.Elems(Before..Obj.Size);
     Obj.Elems(Before) := Elem;
     Obj.Size := Obj.Size + 1;
      end if;
   end Insert;

   procedure Append (Obj : in out Bnd_Node; Elem : Item) is
   begin
      pragma Assert(Obj.Size < Max_Size, "Attempt to insert into full array");
      Obj.Size := Obj.Size + 1;
      Obj.Elems(Obj.Size) := Elem;

   end Append;

   procedure Append (Obj : in out Bnd_Node; Elem : Item;
                          After : Natural) is
   begin
      pragma Assert(After <= Obj.Size, "Exceeded current bounded size");
      if ( After = Obj.Size ) then
     Obj.Size := Obj.Size + 1;
     Obj.Elems(Obj.Size) := Elem;
      else
     Obj.Elems(After + 2 .. Obj.Size + 1) := Obj.Elems(After+1..Obj.Size);
     Obj.Size := Obj.Size + 1;
     Obj.Elems(After+1) := Elem;
      end if;
   end Append;

   procedure Remove (Obj : in out Bnd_Node; From : Natural) is
   begin
      pragma Assert( From <= Obj.Size );
      pragma Assert( Obj.Size > 0 );
      if Obj.Size = 1 then
         Clear(Obj);
      else
     Obj.Elems(From..Obj.Size-1) := Obj.Elems(From+1..Obj.Size);
     Obj.Size := Obj.Size - 1;
      end if;
   end Remove;

   procedure Replace(Obj : in out Bnd_Node; Index : Natural; Elem : Item) is
   begin
      pragma Assert( Index <= Obj.Size );
      Obj.Elems(Index) := Elem;
   end Replace;

   function Available(Obj: Bnd_Node) return Natural is
   begin
      return Max_Size - Obj.Size;
   end Available;

   function Length  (Obj : Bnd_Node) return Natural is
   begin
      return Obj.Size;
   end Length;

   function First   (Obj : Bnd_Node) return Item is
   begin
      pragma Assert( Obj.Size > 0 );
      return Obj.Elems(1);
   end First;

   function Last    (Obj : Bnd_Node) return Item is
   begin
      pragma Assert( Obj.Size > 0 );
      return Obj.Elems(Obj.Size);
   end ;


   function Item_At(Obj : access Bnd_Node; Index : Natural) return Item is
   begin
      pragma Assert(Index > 0);
      return Obj.Elems(Index);
   end Item_At;

   function Location(Obj : access Bnd_Node; Elem : Item;
             Start : Natural := 1) return Natural is
   begin
      for I in Start..Obj.Size loop
         if Obj.Elems(I) = Elem then return I; end if;
      end loop;
      return 0;
   end Location;

end BC.Support.Bounded;
