--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

generic
   type Item is private;
   Maximum_Size : Positive;
package BC.Support.Bounded is

   Max_Size : constant Positive := Maximum_Size;

   type Bnd_Node is private;
	-- An optimally-packed static container whose items are stored on the
	-- stack.  Items are indexable starting at 1.  This is a supporting
	-- type.  As such, it is not intended to be used directly by the
	-- end-user.

   subtype Size_Range is Natural range 0..Max_Size;

   -- Creation, Equality, and Inequality predefined

   procedure Clear  (Obj : in out Bnd_Node);
	-- Empty the container of all Items

   procedure Insert (Obj : in out Bnd_Node; Elem : Item);
	-- Add an item to the front of the container

   procedure Insert (Obj : in out Bnd_Node; Elem : Item; Before : Natural);
	-- Ass an item to the container, before the given index

   procedure Append (Obj : in out Bnd_Node; Elem : Item);
	-- Add an item to the end of the container

   procedure Append (Obj : in out Bnd_Node; Elem : Item; After : Natural);
	-- Add an item to the end of the container, after the given index

   procedure Remove (Obj : in out Bnd_Node; From : Natural);
	-- Remove the item at a given index

   procedure Replace(Obj : in out Bnd_Node; Index : Natural; Elem : Item);
	-- Replace the Item at Index with the new Elem

   function Available(Obj: Bnd_Node) return Natural;
	-- Returns available storage elements

   function Length  (Obj : Bnd_Node) return Natural;
	-- Returns the number of items in the container

   function First   (Obj : Bnd_Node) return Item;
	-- Returns the Item at the front of the container

   function Last    (Obj : Bnd_Node) return Item;
	-- Returns the item at the end of the container

   function Item_At (Obj : access Bnd_Node; Index : Natural) return Item;
	-- Returns the item at the given index

   function Location(Obj : access Bnd_Node; 
							Elem : in Item; Start : in Natural := 1) return Natural;
	-- Returns the first index in which the given item is found. Returns 0
	-- if unsuccessful.

private

   type Elem_Array is array(Positive range <>) of Item;

   type Bnd_Node is record
      Elems : Elem_Array(1..Max_Size);
      Size  : Size_Range := 0;
   end record;

end BC.Support.Bounded;



