--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with BC.Support.Nodes;
with Ada.Finalization;
generic
   type Item is private;
package BC.Support.Dynamic is

   type Dyn_Node is private;
	-- An optimally-packed dynamic container whose items are stored on the heap

   function Create(From : Dyn_Node) return Dyn_Node;
	-- Construct a new Dynamic container that is identical to the given container

   function Create(Size : Positive := 10) return Dyn_Node;
	-- Construct a new dynamic container that has a chunk_Size set to the passed-in value

   function "="(Left, Right : Dyn_Node) return Boolean;
   -- Creation, Equality, and Inequality predefined

   procedure Clear  (Obj : in out Dyn_Node);
  	-- Empty the container of all Items

   procedure Insert (Obj : in out Dyn_Node; Elem : Item);
	-- Add an item to the front of the container

	procedure Insert (Obj : in out Dyn_Node; Elem : Item; Before : Natural);
	-- Ass an item to the container, before the given index

   procedure Append (Obj : in out Dyn_Node; Elem : Item);
	-- Add an item to the end of the container

   procedure Append (Obj : in out Dyn_Node; Elem : Item; After : Natural);
	-- Add an item to the end of the container, after the given index

   procedure Remove (Obj : in out Dyn_Node; From : Natural);
	-- Remove the item at a given index

   procedure Replace(Obj : in out Dyn_Node; Index : Positive; Elem : Item);
	-- Replace the Item at Index with the new Elem

   function Length  (Obj : Dyn_Node) return Natural;
	-- Returns the number of items in the container

   function First   (Obj : Dyn_Node) return Item;
	-- Returns the Item at the front of the container

   function Last    (Obj : Dyn_Node) return Item;
	-- Returns the item at the end of the container

   function Item_At (Obj : Dyn_Node; Index : Positive) return Item;
	-- Returns the item at the given index

   function Location(Obj : Dyn_Node; Elem : Item; Start : Positive := 1) return Natural;
	-- Returns the first index in which the given item is found. Returns 0
	-- if unsuccessful.

   procedure Preallocate(Obj : in out Dyn_Node; New_Length : Natural := 10);
	-- Preallocate New_Length number of unused items for the container

   procedure Set_Chunk_Size(Obj: in out Dyn_Node; Size : Natural);
	-- Set the Chunk_Size for the container

   function Chunk_Size(Obj : in Dyn_Node) return Natural;
	-- returns the current Chunk_Size

private

   use Ada.Finalization;

   type Dyn_Arr is array(Positive range <>) of Item;

   type Dyn_Arr_Ref is access all Dyn_Arr;

   type Dyn_Node is new Controlled with record
      Ref  : Dyn_Arr_Ref;
      Size : Natural := 0;
      Chunk_Size : Natural := 10;
   end record;

   procedure Initialize(Obj : in out Dyn_Node);
   procedure Finalize  (Obj : in out Dyn_Node);
   procedure Adjust    (Obj : in out Dyn_Node);

   Null_Dyn_Node : constant Dyn_Node :=
     (Controlled with Ref => new Dyn_Arr(1..0), Size=> 0, Chunk_Size=> 10 );

end BC.Support.Dynamic;
