--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with Ada.Finalization;
generic
   type Item is private;
package BC.Support.Nodes is

   type Node;
   type Node_Ref is access Node;
   type Node is tagged record
      Next : Node_Ref; 
      Prev : Node_Ref;
      Element : Item;
   end record;

   type Single_Node;
   type Single_Node_Ref is access Single_Node;
   type Single_Node is record
      Elem  : aliased Item;
      Next  : Single_Node_Ref; 
      Count : Natural :=1;
   end record;

   type Double_Node;
   type Double_Node_Ref is access Double_Node;
   type Double_Node is record
      Elem  : aliased Item;
      Prev  : Double_Node_Ref;
      Next  : Double_Node_Ref; 
      Count : Natural :=1;
   end record;


-- Class denoting a simple node consisting of an item, a pointer to
-- the parent, pointers to the left and right items, and a reference count
   type Binary_Node;
   type Binary_Node_Ref is access Binary_Node;

   type Binary_Node is record
      Elem : aliased Item;
      Parent, Left, Right : Binary_Node_Ref;
      Count : Natural := 1;
   end record;


-- Class denoting a simple node consisting of an item, a pointer to
-- the parent, pointers to the child and sibling items, and a reference count
   type Multiway_Node;
   type Multiway_Node_Ref is access Multiway_Node;

   type Multiway_Node is record
      Elem : aliased Item;
      Parent, Child, Sibling : Multiway_Node_Ref;
      Count : Natural := 1;
   end record;

-- Class denoting a simple node consisting of an item, pointers to the
-- left and right items, and a balance
   type AVL_Node;
   type AVL_Node_Ref is access AVL_Node;
   type Balance is (Left, Middle, Right);

   type AVL_Node is record
      Elem : aliased Item;
      Left, Right : AVL_Node_Ref;
      Bal : Balance := Middle;
   end record;

end BC.Support.Nodes;
