--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with Ada.Unchecked_Deallocation;
package body BC.Support.Unbounded is

   use type Nodes.Node_Ref;

   procedure Free is new
     Ada.Unchecked_Deallocation(Nodes.Node, Nodes.Node_Ref);

   function Create(From : Unb_Node) return Unb_Node is
      Obj : Unb_Node := From;
      Tmp : Nodes.Node_Ref := Obj.Rep;
      Cache_Cow : Natural := 0;
   begin
      while Tmp /= null loop
         Tmp := new Nodes.Node'(Tmp.all);
         Cache_Cow := Cache_Cow + 1;
         Tmp := Tmp.Next;
      end loop;
      Obj.Last := Tmp; Obj.Cache := Tmp; Obj.Cache_Index := Cache_Cow;
      return Obj;
   end Create;

   function "="(Left, Right : in Unb_Node) return Boolean is
   begin
      if Left.Size = Right.Size then
         declare
            Temp_L : Nodes.Node_Ref := Left.Rep;
            Temp_R : Nodes.Node_Ref := Right.Rep;
         begin
            while Temp_L /= null loop
               if Temp_L.Element /= Temp_R.Element then
                  return False;
               end if;
               Temp_L := Temp_L.Next;
               Temp_R := Temp_R.Next;
            end loop;
            return True;
         end;
      else
         return False;
      end if;
   end "=";

   procedure Clear  (Obj : in out Unb_Node) is
      Empty_Node : Unb_Node;
      Ptr : Nodes.Node_Ref;
   begin
      while Obj.Rep /= null loop
         Ptr := Obj.Rep;
         Obj.Rep := Obj.Rep.Next;
         Free(Ptr);
      end loop;
      Obj := Empty_Node;
   end Clear;

   procedure Insert(Obj : in out Unb_Node; Elem : Item) is
   begin
      Obj.Rep := new Nodes.Node'(Obj.Rep, null, Elem);
      if Obj.Last.Prev /= null then
         Obj.Last.Prev.Next := Obj.Last;
      end if;
      if Obj.Last = null then
         Obj.Last := Obj.Rep;
      end if;
      Obj.Size := Obj.Size + 1;
      Obj.Cache := Obj.Rep;
      Obj.Cache_Index := 1;
   end Insert;

   procedure Insert(Obj : in out Unb_Node; Elem : Item; Before : Natural) is
   begin
      pragma Assert(Before <= Obj.Size);
      if (Obj.Size = 0) or else (Before <= 1) then
         Insert(Obj,Elem);
      else
         declare
        AObj      : aliased Unb_Node := Obj;
            Temp_Item : Item := Item_At(AObj'access, Before);  -- loads cache
            Temp_Node : Nodes.Node_Ref;
         begin
            Temp_Node := new Nodes.Node'(Obj.Cache, Obj.Cache.Prev, Elem);
      if Temp_Node.Prev /= null then
         Temp_Node.Prev.Next := Temp_Node;
      else
               Obj.Rep := Temp_Node;
        end if;
        Obj.Size := Obj.Size + 1;
        Obj.Cache := Temp_Node;
     end;
      end if;
   end Insert;

   procedure Append (Obj : in out Unb_Node; Elem : Item) is
   begin
      Obj.Last := new Nodes.Node'(null, Obj.Last, Elem);
      if Obj.Last.Prev /= null then
         Obj.Last.Prev.Next := Obj.Last;
      end if;
      if Obj.Rep = null then
         Obj.Rep := Obj.Last;
      end if;
      Obj.Size := Obj.Size + 1;
      Obj.Cache := Obj.Last;
      Obj.Cache_Index := Obj.Size;
   end Append;

   procedure Append (Obj : in out Unb_Node; Elem : Item; After : Natural) is
   begin
      pragma Assert(After <= Obj.Size);
      if (Obj.Size = 0) or else (After <= 1) then
         Append(Obj,Elem);
      else
         declare
        AObj      : aliased Unb_Node := Obj;
            Temp_Item : Item := Item_At(AObj'access, After);  -- loads cache
            Temp_Node : Nodes.Node_Ref;
        begin
            Temp_Node := new Nodes.Node'(Obj.Cache.Next, Obj.Cache, Elem);
      if Temp_Node.Prev /= null then
         Temp_Node.Prev.Next := Temp_Node;
      end if;
            if Temp_Node.Next = null then
               Obj.Last := Temp_Node;
            end if;
            Obj.Size := Obj.Size + 1;
            Obj.Cache := Temp_Node;
            Obj.Cache_Index := Obj.Cache_Index + 1;
        end;
      end if;
   end Append;

   procedure Remove (Obj : in out Unb_Node; From : Natural) is
   begin
      pragma Assert( From <= Obj.Size );
      pragma Assert( Obj.Size > 0 );
      if Obj.Size = 1 then
         Clear(Obj);
      else
         declare
        AObj      : aliased Unb_Node := Obj;
            Temp_Item : Item := Item_At(AObj'access, From);
            Ptr : Nodes.Node_Ref := AObj.Cache;
         begin
            if Ptr.Prev = null then
               AObj.Rep := Ptr.Next;
            else
               Ptr.Prev.Next := Ptr.Next;
            end if;
            if Ptr.Next = null then
               AObj.Last := Ptr.Prev;
            else
               Ptr.Next.Prev := Ptr.Prev;
            end if;
            AObj.Size := AObj.Size - 1;
            if Ptr.Next /= null then
               AObj.Cache := Ptr.Next;
            elsif Ptr.Prev /= null then
               AObj.Cache := Ptr.Prev;
               AObj.Cache_Index := AObj.Cache_Index - 1;
            else
               AObj.Cache := null;
               AObj.Cache_Index := 0;
            end if;
            Free(Ptr);
            Obj := AObj;
         end;
      end if;
   end Remove;

   procedure Replace(Obj : in out Unb_Node; Index : Positive; Elem : Item) is
   begin
      pragma Assert( Index <= Obj.Size );
      if not ( (obj.Cache /= null) and then (Index = Obj.Cache_Index)) then
         declare
            Ptr : Nodes.Node_Ref := Obj.Rep;
         begin
            for i in 1..Obj.Size loop
               if i = Index then
                  Obj.Cache := Ptr;
                  Obj.Cache_Index := i;
                  exit;
               else
                  Ptr := Ptr.Next;
               end if;
            end loop;
         end;
      end if;
      Obj.Cache.Element := Elem;
   end Replace;

   function Length  (Obj : Unb_Node) return Natural is
   begin
      return Obj.Size;
   end Length;

   function First   (Obj : Unb_Node) return Item is
   begin
      pragma Assert( Obj.Size > 0 );
      return Obj.Rep.Element;
   end First;

   function Last    (Obj : Unb_Node) return Item is
   begin
      pragma Assert( Obj.Size > 0 );
      return Obj.Last.Element;
   end ;


   function Item_At(Obj : access Unb_Node; Index : Positive) return Item is
   begin
      if Obj.Cache /= null then
         if Index = Obj.Cache_Index then
            return Obj.Cache.Element;
         elsif Index = Obj.Cache_Index + 1 then
            Obj.Cache := Obj.Cache.Next;
            Obj.Cache_Index := Obj.Cache_Index + 1;
            return Obj.Cache.Element;
         elsif Index = Obj.Cache_Index - 1 then
            Obj.Cache := Obj.Cache.Prev;
            Obj.Cache_Index := Obj.Cache_Index - 1;
            return Obj.Cache.Element;
         end if;
      end if;
      declare
         Ptr : Nodes.Node_Ref := Obj.Rep;
      begin
         for i in 1..Obj.Size loop
            if i = Index then
               Obj.Cache := Ptr;
               Obj.Cache_Index := i;
               return Ptr.Element;
            else
               Ptr := Ptr.Next;
            end if;
         end loop;
         return Ptr.Element;
      end;
   end Item_At;

   function Location(Obj : access Unb_Node; Elem : Item; Start : Positive := 1) return Natural is
      Ptr : Nodes.Node_Ref := Obj.Rep;
   begin
      pragma Assert( Start < Obj.Size );
      if (Start = Obj.Cache_Index) and then (Elem = Obj.Cache.Element) then
         return Obj.Cache_Index;
      end if;
      for i in 1..Start-1 loop
         Ptr := Ptr.Next; -- advance to Start point
      end loop;
      for i in Start..Obj.Size loop
         if Ptr.Element = Elem then
            Obj.Cache := Ptr;
            Obj.Cache_Index := i;
            return i;
         else
            Ptr := Ptr.Next;
         end if;
      end loop;
      return 0;
   end Location;

end BC.Support.Unbounded;
