--  The Ada 95 Booch Components (Version 0.4q)
--  (C) Copyright 1987-1994 Grady Booch.
--  (C) Copyright 1994-1996 Grady Booch and David Weller.
--
-- The Ada 95 Booch Components is a collection of free software components.
-- You can redistribute it and/or modify it under terms of the Ada
-- Community License.
--

with Text_Io;
with Queue_Root;
with Queue_Bounded;
with Queue_Dynamic;
with Queue_Unbounded;
procedure Queue_Test is
   use Text_IO; use Queue_Root;
   use Queue_Bounded; use Queue_Dynamic; use Queue_Unbounded;

   procedure Process (C : in out Character; OK : out Boolean) is
      use Text_IO;
   begin
      Put_Line("Item: " & C );
      Ok := True;
   end Process;

   procedure Apply is new Apply(Apply_Method=> Process);

   procedure Assertion ( Cond : Boolean; Message : String) is
   begin
      if not Cond then
     Put_Line(Message);
      end if;
   end Assertion;
   pragma Inline(Assertion);

   procedure Test_Active_Iterator (L : access Queue'Class) is
      Iter : Iterator(L);
      Success : Boolean;
   begin
      while not Is_Done(Iter) loop
     Process(Current_Item(Iter).all, Success);
     Next(Iter);
      end loop;
   end Test_Active_Iterator;

   procedure Test_Primitive(Q1, Q2 : in out Queue'Class) is
   begin
      Assertion(Is_Empty(Q1), "** P01: Queue is not initially empty");
      Assertion(Length(Q1) = 0, "** P02: Queue length is not initially zero");
      Append(Q1, '1');
      Append(Q1, '2');
      Append(Q1, '3');
      Assertion(not (Is_Empty(Q1)), "** P03: Queue is empty");
      Assertion((Length(Q1) = 3), "** P04: Queue length is not correct");
      Assertion((Front(Q1) = '1'), "** P05: Queue front is not correct");
      Clear(Q1);
      Assertion(Is_Empty(Q1), "** P06: Queue is not empty");
      Assertion((Length(Q1) = 0), "** P07: Queue length is not zero");
      Append(Q1, '4');
      Append(Q1, '5');
      Append(Q1, '6');
      Assertion(not (Is_Empty(Q1)), "** P08: Queue is empty");
      Assertion((Length(Q1) = 3), "** P09: Queue length is not correct");
      Assertion((Front(Q1) = '4'), "** P10: Queue front is not correct");
      Pop(Q1);
      Pop(Q1);
      Assertion(not (Is_Empty(Q1)), "** P11: Queue is empty");
      Assertion((Length(Q1) = 1), "** P12: Queue length is not correct");
      Assertion((Front(Q1) = '6'), "** P13: Queue front is not correct");
      Pop(Q1);
      Assertion(Is_Empty(Q1), "** P14: Queue is not empty");
      Assertion((Length(Q1) = 0), "** P15: Queue length is not zero");
      Append(Q1, '7');
      Append(Q1, '8');
      Append(Q1, '9');
      Pop(Q1);
      Pop(Q1);
      Assertion(not (Is_Empty(Q1)), "** P16: Queue is empty");
      Assertion((Length(Q1) = 1), "** P17: Queue length is not correct");
      Assertion((Front(Q1) = '9'), "** P18: Queue front is not correct");
      Q2:= Q1;
      Assertion(not (Is_Empty(Q1)), "** P19: Queue is empty");
      Assertion((Length(Q1) = 1), "** P20: Queue length is not correct");
      Assertion((Front(Q1) = '9'), "** P21: Queue front is not correct");
      Assertion(not (Is_Empty(Q2)), "** P22: Queue is empty");
      Assertion((Length(Q2) = 1), "** P23: Queue length is not correct");
      Assertion((Front(Q2) = '9'), "** P24: Queue front is not correct");
      Assertion((q1 = q2), "** P25: Queues are not equal");
      Clear(Q2);
      Assertion( ( not(Is_Empty(Q1)) ) , "** P26: Queue is empty");
      Assertion(Is_Empty(Q2), "** P29: Queue is not empty");
      Assertion((Length(Q1) = 1), "** P27: Queue length is not correct");
      Assertion((Front(Q1) = '9'), "** P28: Queue front is not correct");
      Assertion((Length(Q2) = 0), "** P30: Queue length is not correct");
      Assertion((q1 /= q2), "** P31: Queues not equal");
      pragma Debug(Put_Line("Pre-Append: Expecting(from q1): 9"));
      pragma Debug(Test_Active_Iterator(Q1'access));
      pragma Debug(Put_Line("Pre-Append: Expecting(from q2): nothing"));
      pragma Debug(Test_Active_Iterator(Q2'access));
      Append(Q2, '1');
      pragma Debug(Put_Line("Post-Append: Expecting(from q1): 9"));
      pragma Debug(Test_Active_Iterator(Q1'access));
      pragma Debug(Put_Line("Post-Append: Expecting(from q1): 1"));
      pragma Debug(Test_Active_Iterator(Q2'access));
      Append(Q2, '2');
      Append(Q2, '3');
      Append(Q2, '4');
      Assertion(Location(Q2, '1') = 1, "** P32: Queue location is not correct");
      Assertion(Location(Q2, '2') = 2, "** P33: Queue location is not correct");
      Assertion(Location(Q2, '4') = 4, "** P34: Queue location is not correct");
      Remove(Q2, 1);
      Remove(Q2, 2);
      Remove(Q2, 2);
      Assertion((Length(Q2) = 1), "** P35: Queue length is not correct");
      Assertion((Front(Q2) = '2'), "** P36: Queue front is not correct");
      Remove(Q2, 1);
      Assertion((Length(Q2) = 0), "** P37: Queue length is not correct");
   end Test_Primitive;

   procedure Test_Passive_Iterator (L : access Queue'Class) is
      PIter : Passive_Iterator(L);
      Success : Boolean;
   begin
      Apply(PIter, Success);
   end Test_Passive_Iterator;

   queue_b_p1, Queue_B_P2 : aliased Bnd_Queue;
   queue_d_p1, queue_d_p2 : aliased Dyn_Queue;
   queue_u_p1, Queue_U_P2 : aliased Unb_Queue;

begin
  Put_Line("Starting queue tests");

  Put_Line("...Bounded Queue");
  test_primitive(queue_b_p1, queue_b_p2);

  Put_Line("...Dynamic Queue");
  Preallocate(Queue_D_P1, 50);
  test_primitive(queue_d_p1, queue_d_p2);

  Put_Line("...Unbounded Queue");
  test_primitive(queue_u_p1, queue_u_p2);

  Put_Line("...Queue Active Iterator");
  Put_Line("   Bounded:");
  test_active_iterator(Queue_B_P1'access);
  Put_Line("   Dynamic:");
  test_active_iterator(Queue_D_P1'access);
  Put_Line("   Unbounded:");
  test_active_iterator(Queue_U_P1'access);

  Put_Line("...Queue Passive Iterator");
  Put_Line("   Bounded:");
  test_passive_iterator(Queue_B_P1'access);
  Put_Line("   Dynamic:");
  test_passive_iterator(Queue_D_P1'access);
  Put_Line("   Unbounded:");
  test_passive_iterator(Queue_U_P1'Access);

  Assertion((Front(Queue_B_p1) = '9'), "** M01: Queue front is not correct");
  Assertion((Length(Queue_B_p2) = 0), "** M02: Queue length is not correct");
  Assertion((Front(Queue_D_p1) = '9'), "** M05: Queue front is not correct");
  Assertion((Length(Queue_D_p2) = 0), "** M06: Queue length is not correct");
  Assertion((Front(Queue_U_p1) = '9'), "** M09: Queue front is not correct");
  Assertion((Length(Queue_U_p2) = 0), "** M10: Queue length is not correct");
  Assertion((Available(Queue_b_p1) = 99),
    "** M13: Available space is not correct");
  Assertion((Available(Queue_B_p2) = 100),
    "** M14: Available space is not correct");

  Put_Line("Completed queue tests");

end Queue_Test;




